/**
* Copyright (c) Huawei Technologies Co., Ltd. 2020-2022. All rights reserved.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at

* http://www.apache.org/licenses/LICENSE-2.0

* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.

* File sample_process.h
* Description: handle acl resource
*/
#ifndef DETECTPOSTPROCESSTHREAD_H
#define DETECTPOSTPROCESSTHREAD_H
#pragma once

#include <iostream>
#include <mutex>
#include <unistd.h>
#include "acl/acl.h"
#include "AclLiteError.h"
#include "AclLiteImageProc.h"
#include "AclLiteThread.h"
#include "CarParams.h"

class DetectPostprocessThread : public AclLiteThread {
public:
    DetectPostprocessThread();
//    DetectPostprocessThread(ModelInfo programinfo);

    DetectPostprocessThread(const char *&configFile, int channelId, ModelInfo programinfo);

    ~DetectPostprocessThread();

    AclLiteError Init();
    AclLiteError Process(int msgId, std::shared_ptr<void> data);
    std::vector<BBoxstr> nmsAllClasses(const float nmsThresh, std::vector<BBoxstr>& binfo, const uint numClasses);
    std::vector<BBoxstr> nonMaximumSuppression(const float nmsThresh, std::vector<BBoxstr> binfo);
    virtual std::vector<BBoxstr> outPutDecode(float* detectData, ModelInfo modelinfo, int image_ori_width,int image_ori_height,int image_resize_width,int image_resize_height);
    AclLiteError GetThreshold(float &threshold, uint32_t channelId);
    virtual AclLiteError InferOutputProcess(std::shared_ptr<ObjDetectDataMsg> objDetectDataMsg);
private:

    AclLiteError MsgSend(std::shared_ptr<ObjDetectDataMsg> objDetectDataMsg);

public:
    float threshold_ = 0.3;
    const char* configFile_;
    uint32_t totalBox_ = 17640;
    int channelId_;
    ModelInfo modelInfo_;


};
class onlyDetectPostprocessThread : public AclLiteThread {
public:
    onlyDetectPostprocessThread();
    onlyDetectPostprocessThread(const char*& configFile, int channelId,ModelInfo programinfo);
    ~onlyDetectPostprocessThread();

    AclLiteError Init();
    AclLiteError Process(int msgId, std::shared_ptr<void> data);
    std::vector<BBoxstr> nmsAllClasses(const float nmsThresh, std::vector<BBoxstr>& binfo, const uint numClasses);
    std::vector<BBoxstr> nonMaximumSuppression(const float nmsThresh, std::vector<BBoxstr> binfo);
    AclLiteError GetOutputFrameResolution(int& frameWidth, int& frameHeight, uint32_t channelId);

    ModelInfo modelInfo_;
    uint32_t totalBox_ = 17640;
    int channelId_;
    float threshold_ = 0.3;

private:
    const uint32_t kLineSolid_ = 2;
    int programId_;
    const uint32_t kLabelOffset_ = 11;
    const double kFountScale_ = 0.5;
    const cv::Scalar kFontColor_ = (0, 0, 255);
    const std::vector<cv::Scalar> kColors_ {
            cv::Scalar(237, 149, 100),
            cv::Scalar(0, 215, 255),
            cv::Scalar(50, 205, 50),
            cv::Scalar(139, 85, 26) };

    AclLiteError GetOutputDataType(std::string& outputType, uint32_t channelId);
    AclLiteError SetOutputVideo();
    AclLiteError SendImage(std::shared_ptr<ObjDetectDataMsg> &objDetectDataMsg);
    AclLiteError DisplayMsgSend(std::shared_ptr<ObjDetectDataMsg> objDetectDataMsg);
    AclLiteError PrintResult(std::shared_ptr<ObjDetectDataMsg> &objDetectDataMsg);
    AclLiteError DrawResultOnPic(std::shared_ptr<ObjDetectDataMsg> &objDetectDataMsg);
    AclLiteError DrawResultOnVideo(std::shared_ptr<ObjDetectDataMsg> &objDetectDataMsg);
    AclLiteError InferOutputProcess(std::shared_ptr<ObjDetectDataMsg> objDetectDataMsg);
    std::vector<BBoxstr> outPutDecode(float* detectData, ModelInfo modelinfo, int image_ori_width,int image_ori_height,int image_resize_width,int image_resize_height);



private:

    const char* configFile_;
    std::string outputType_;
    int outputFrameWidth_;
    int outputFrameHeight_;
    cv::VideoWriter outputVideo_;

    AclLiteError GetThreshold(float &threshold, uint32_t channelId);
};


#endif